@@ -5,17 +5,22 @@ use Parse::BBCode::Tag;
 use Parse::BBCode::HTML qw/ &defaults &default_escapes &optional /;
 use base 'Class::Accessor::Fast';
 __PACKAGE__->follow_best_practice;
-__PACKAGE__->mk_accessors(qw/ tags allowed compiled plain strict_attributes
-    close_open_tags error tree escapes direct_attribute /);
-use Data::Dumper;
+__PACKAGE__->mk_accessors(qw/
+    tags allowed compiled plain strict_attributes close_open_tags error
+    tree escapes direct_attribute params url_finder text_processor linebreaks
+    smileys /);
+#use Data::Dumper;
 use Carp;
 my $scalar_util = eval "require Scalar::Util; 1";
 
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 
 my %defaults = (
     strict_attributes   => 1,
     direct_attribute    => 1,
+    linebreaks          => 1,
+    smileys             => 0,
+    url_finder          => 0,
 );
 sub new {
     my ($class, $args) = @_;
@@ -38,6 +43,7 @@ sub new {
         %args
     });
     $self->set_allowed([ grep { length } keys %{ $self->get_tags } ]);
+    $self->_compile_tags;
     return $self;
 }
 
@@ -62,7 +68,8 @@ sub permit {
 
 sub _compile_tags {
     my ($self) = @_;
-    unless ($self->get_compiled) {
+#    unless ($self->get_compiled) {
+    {
         my $defs = $self->get_tags;
 
         # get definition for how text should be rendered which is not in tags
@@ -74,11 +81,99 @@ sub _compile_tags {
             }
         }
         else {
-            $plain = sub {
-                my $text = Parse::BBCode::escape_html($_[2]);
-                $text =~ s/\r?\n|\r/<br>\n/g;
-                $text;
-            };
+            my $url_finder = $self->get_url_finder;
+            my $linebreaks = $self->get_linebreaks;
+            my $smileys = $self->get_smileys;
+            if ($url_finder) {
+                my $result = eval { require URI::Find; 1 };
+                unless ($result) {
+                    undef $url_finder;
+                }
+            }
+            my $escape = \&Parse::BBCode::escape_html;
+            my $post_processor_1 = $escape;
+            my $post_processor;
+            my $text_processor = $self->get_text_processor;
+            if ($text_processor) {
+                $post_processor_1 = $text_processor;
+            }
+            if ($smileys and ref($smileys->{icons}) eq 'HASH') {
+                $smileys = {
+                    icons => $smileys->{icons},
+                    base_url => $smileys->{base_url} || '/smileys/',
+                    format => $smileys->{format} || '<img src="%s" alt="%s">',
+                };
+                my $re = join '|', map { quotemeta $_ } sort { length $b <=> length $a }
+                    keys %{ $smileys->{icons} };
+                my $code = sub {
+                    my ($text, $post_processor) = @_;
+                    my $out = '';
+                    while ($text =~ s/^ (.*?[\s]|^) ($re) (?=[\s]|$)//xsm) {
+                        my ($pre, $emo) = ($1, $2);
+                        my $url = "$smileys->{base_url}$smileys->{icons}->{$emo}";
+                        my $emo_escaped = Parse::BBCode::escape_html($emo);
+                        my $image_tag = sprintf $smileys->{format}, $url, $emo_escaped;
+                        $out .= $post_processor_1->($pre) . $image_tag;
+                    }
+                    $out .= $post_processor_1->($text);
+                    return $out;
+                };
+                $post_processor = $code;
+            }
+            else {
+                $post_processor = $post_processor_1;
+            }
+
+            if ($url_finder) {
+                my $url_find_sub;
+                if (ref($url_finder) eq 'CODE') {
+                    $url_find_sub = $url_finder;
+                }
+                else {
+                    unless (ref($url_finder) eq 'HASH') {
+                        $url_finder = {
+                            max_length => 50,
+                            format => '<a href="%s" rel="nofollow">%s</a>',
+                        };
+                    }
+                    my $max_url = $url_finder->{max_length} || 0;
+                    my $format = $url_finder->{format};
+                    my $finder = URI::Find->new(sub {
+                        my ($url) = @_;
+                        my $title = $url;
+                        if ($max_url and length($title) > $max_url) {
+                            $title = substr($title, 0, $max_url) . "...";
+                        }
+                        my $escaped = Parse::BBCode::escape_html($url);
+                        my $escaped_title = Parse::BBCode::escape_html($title);
+                        my $href = sprintf $format, $escaped, $title;
+                        return $href;
+                    });
+                    $url_find_sub = sub {
+                        my ($ref_content, $post, $info) = @_;
+                        $finder->find($ref_content, sub { $post->($_[0], $info) });
+                    };
+                }
+                $plain = sub {
+                    my ($parser, $attr, $content, $info) = @_;
+                    unless ($info->{classes}->{url}) {
+                        $url_find_sub->(\$content, $post_processor, $info);
+                    }
+                    else {
+                        $content = $post_processor->($content);
+                    }
+                    $content =~ s/\r?\n|\r/<br>\n/g if $linebreaks;
+                    $content;
+                };
+            }
+            else {
+                $plain = sub {
+                    my ($parser, $attr, $content, $info) = @_;
+                    my $text = $post_processor->($content, $info);
+                    $text =~ s/\r?\n|\r/<br>\n/g if $linebreaks;
+                    $text;
+                };
+            }
             $self->set_plain($plain);
         }
 
@@ -95,6 +190,8 @@ sub _compile_tags {
                 $defs->{$key} = $new_def;
             }
             $defs->{$key}->{class} ||= 'inline';
+            $defs->{$key}->{classic} = 1 unless defined $defs->{$key}->{classic};
+            $defs->{$key}->{close} = 1 unless defined $defs->{$key}->{close};
         }
         $self->set_compiled(1);
     }
@@ -184,7 +281,7 @@ sub _compile_def {
                             # if escape returns undef, we return it unparsed
                             return $tag->get_start
                                 . (join '', map {
-                                    $self->render_tree($_);
+                                    $self->_render_tree($_);
                                 } @{ $tag->get_content })
                                 . $tag->get_end;
                         }
@@ -210,13 +307,16 @@ sub _render_text {
 }
 
 sub parse {
-    my ($self, $text) = @_;
-    $self->set_error({});
-    $self->_compile_tags;
+    my ($self, $text, $params) = @_;
+    $self->set_error(undef);
     my $defs = $self->get_tags;
     my $tags = $self->get_allowed || [keys %$defs];
-    my $re = join '|', map { quotemeta } sort {length $b <=> length $a } @$tags;
-    $re = qr/$re/i;
+    my @classic_tags = grep { $defs->{$_}->{classic} } @$tags;
+    my @short_tags = grep { $defs->{$_}->{short} } @$tags;
+    my $re_classic = join '|', map { quotemeta } sort {length $b <=> length $a } @classic_tags;
+    #$re_classic = qr/$re_classic/i;
+    my $re_short = join '|', map { quotemeta } sort {length $b <=> length $a } @short_tags;
+    #$re_short = qr/$re_short/i;
     #warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$re], ['re']);
     my @tags;
     my $out = '';
@@ -284,12 +384,32 @@ sub parse {
         $in_url = grep { $_->get_class eq 'url' } @opened;
         #warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$in_url], ['in_url']);
         #warn __PACKAGE__.':'.__LINE__.": ============= match $text\n";
-        my ($before, $tag, $after) = split m{ \[ ($re) (?=\b|\]|\=) }x, $text, 2;
-        #warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\@opened], ['opened']);
+        my $tag;
+        my ($before, $tag1, $tag2, $after);
+        if ($re_classic and $re_short) {
+            ($before, $tag1, $tag2, $after) = split m{
+                (?:
+                    \[ ($re_short)   (?=://)
+                    |
+                    \[ ($re_classic) (?=\b|\]|\=)
+                )
+            }ix, $text, 2;
+        }
+        elsif (! $re_classic and $re_short) {
+            ($before, $tag1, $after) = split m{
+                    \[ ($re_short)   (?=://)
+            }ix, $text, 2;
+        }
+        elsif ($re_classic and !$re_short) {
+            ($before, $tag2, $after) = split m{
+                    \[ ($re_classic) (?=\b|\]|\=)
+            }ix, $text, 2;
+        }
         { no warnings;
-        #warn __PACKAGE__.':'.__LINE__.": $before, $tag, $after)\n";
+#            warn __PACKAGE__.':'.__LINE__.": $before, $tag1, $tag2, $after)\n";
         #warn __PACKAGE__.':'.__LINE__.": RE: $current_open_re\n";
         }
+        #warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\@opened], ['opened']);
         if (length $before) {
             # look if it contains a closing tag
             #warn __PACKAGE__.':'.__LINE__.": BEFORE $before\n";
@@ -337,9 +457,43 @@ sub parse {
                     $callback_found_tag->($f);
                 }
             }
-            #warn __PACKAGE__." === before=$before ($tag)\n";
+#            warn __PACKAGE__." === before='$before' ($tag)\n";
             $callback_found_text->($before);
         }
+
+        if (defined $tag1) {
+            # short tag
+#            $callback_found_text->($before) if length $before;
+            if ($after =~ s{ :// ([^\[]+) \] }{}x) {
+                my $content = $1;
+                my ($attr, $title) = split /\|/, $content, 2;
+                my $tag = Parse::BBCode::Tag->new({
+                        name    => lc $tag1,
+                        attr    => [[$attr]],
+                        content => [(defined $title and length $title) ? $title : ()],
+                        start   => "[$tag1://$content]",
+                        close   => 0,
+                        class   => $defs->{lc $tag1}->{class},
+                        single  => $defs->{lc $tag1}->{single},
+                        in_url  => $in_url,
+                        type    => 'short',
+                    });
+                if ($in_url and $tag->get_class eq 'url') {
+                    $callback_found_text->($tag->get_start);
+                }
+                else {
+                    $callback_found_tag->($tag);
+                }
+            }
+            else {
+                $callback_found_text->("[$tag1");
+            }
+            $text = $after;
+            next;
+        }
+        $tag = $tag2;
+
+
         $in_url = grep { $_->get_class eq 'url' } @opened;
 
         if ($after) {
@@ -353,18 +507,30 @@ sub parse {
                 my $attr = $1;
                 $attr = '' unless defined $attr;
                 #warn __PACKAGE__.':'.__LINE__.": found attribute for $tag: $attr\n";
+                my $close = $defs->{lc $tag}->{close};
+                my $def = $defs->{lc $tag};
                 my $open = Parse::BBCode::Tag->new({
                         name    => lc $tag,
                         attr    => [],
                         content => [],
                         start   => "[$tag$attr]",
-                        close   => $defs->{lc $tag}->{close},
+                        close   => $close,
                         class   => $defs->{lc $tag}->{class},
                         single  => $defs->{lc $tag}->{single},
                         in_url  => $in_url,
+                        type    => 'classic',
                     });
                 my $success = $self->_validate_attr($open, $attr);
                 my $nested_url = $in_url && $open->get_class eq 'url';
+                if ($success) {
+                    my $last = $opened[-1];
+                    if ($last and not $last->get_close and not $close) {
+                        $self->_finish_tag($last, '');
+                        # tag which should not have closing tag
+                        pop @opened;
+                        $callback_found_tag->($last);
+                    }
+                }
                 if ($success && $open->get_single && !$nested_url) {
                     $self->_finish_tag($open, '');
                     $callback_found_tag->($open);
@@ -414,7 +580,7 @@ sub parse {
         #sleep 1;
         #warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\@tags], ['tags']);
     }
-    #warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\@opened], ['opened']);
+#    warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\@opened], ['opened']);
     if ($self->get_close_open_tags) {
         while (my $opened = pop @opened) {
             $self->_add_error('unclosed', $opened);
@@ -424,7 +590,6 @@ sub parse {
     }
     else {
         while (my $opened = shift @opened) {
-            #warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$opened], ['opened']);
             my @text = $opened->_reduce;
             push @tags, @text;
         }
@@ -432,6 +597,10 @@ sub parse {
     if ($scalar_util) {
         Scalar::Util::weaken($callback_found_tag);
     }
+    else {
+        # just to make sure no memleak if there's no Scalar::Util
+        undef $callback_found_tag;
+    }
     #warn __PACKAGE__.':'.__LINE__.": !!!!!!!!!!!! left text: '$text'\n";
     #warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\@tags], ['tags']);
     my $tree = Parse::BBCode::Tag->new({
@@ -441,8 +610,7 @@ sub parse {
         class => 'block',
         attr => [[]],
     });
-    # just to make sure no memleak if there's no Scalar::Util
-    undef $callback_found_tag;
+    $tree->_init_info({});
     return $tree;
 }
 
@@ -466,14 +634,14 @@ sub error {
 }
 
 sub render {
-    my ($self, $text) = @_;
+    my ($self, $text, $params) = @_;
     if (@_ < 2) {
         croak ("Missing input - Usage: \$parser->render(\$text)");
     }
     #warn __PACKAGE__.':'.__LINE__.": @_\n";
     #sleep 2;
-    my $tree = $self->parse($text);
-    my $out = $self->render_tree($tree);
+    my $tree = $self->parse($text, $params);
+    my $out = $self->render_tree($tree, $params);
     if ($self->get_error) {
         $self->set_tree($tree);
     }
@@ -481,6 +649,15 @@ sub render {
 }
 
 sub render_tree {
+    my ($self, $tree, $params) = @_;
+    $params ||= {};
+    $self->set_params($params);
+    my $rendered = $self->_render_tree($tree);
+    $self->set_params(undef);
+    return $rendered;
+}
+
+sub _render_tree {
     my ($self, $tree, $outer, $info) = @_;
     my $out = '';
     $info ||= {
@@ -496,7 +673,7 @@ sub render_tree {
         my @stack = @{ $info->{stack} };
         push @stack, $name;
         my %classes = %{ $info->{classes} };
-        $classes{ $tree->get_class }++;
+        $classes{ $tree->get_class || '' }++;
         my %info = (
             tags => \%tags,
             stack => [@stack],
@@ -504,10 +681,18 @@ sub render_tree {
         );
         my $code = $defs->{$name}->{code};
         my $parse = $defs->{$name}->{parse};
-        my $attr = $tree->get_attr->[0]->[0];
+        my $attr = $tree->get_attr || [];
+        $attr = $attr->[0]->[0];
         my $content = $tree->get_content;
-        my $fallback = (defined $attr and length $attr) ? $attr : $content;
+        my $fallback;
         my $string = '';
+        if (($tree->get_type || 'classic') eq 'classic') {
+            $fallback = (defined $attr and length $attr) ? $attr : $content;
+        }
+        else {
+            $fallback = $attr;
+            $string = @$content ? '' : $attr;
+        }
         if (ref $fallback) {
             # we have recursive content, we don't want that in
             # an attribute
@@ -515,7 +700,7 @@ sub render_tree {
                 not ref $_
             } @$fallback;
         }
-        if ($tree->get_class eq 'block') {
+        if (($tree->get_class || '') eq 'block') {
             if (@$content == 1 and not ref $content->[0] and defined $content->[0]) {
                 $content->[0] =~ s/^\r?\n//;
                 $content->[0] =~ s/\r?\n\z//;
@@ -531,7 +716,7 @@ sub render_tree {
         }
         if (not exists $defs->{$name}->{parse} or $parse) {
             for my $c (@$content) {
-                $string .= $self->render_tree($c, $tree, \%info);
+                $string .= $self->_render_tree($c, $tree, \%info);
             }
         }
         else {
@@ -614,17 +799,26 @@ __END__
 
 =head1 NAME
 
-Parse::BBCode - Module to turn BBCode into HTML or plain text
+Parse::BBCode - Module to parse BBCode and render it as HTML or text
 
 =head1 SYNOPSIS
 
 To parse a bbcode string, set up a parser with the default HTML defintions
 of L<Parse::BBCode::HTML>:
 
+    # render bbcode to HTML
     use Parse::BBCode;
     my $p = Parse::BBCode->new();
     my $code = 'some [b]b code[/b]';
-    my $parsed = $p->render($code);
+    my $rendered = $p->render($code);
+
+    # parse bbcode, manipulate tree and render
+    use Parse::BBCode;
+    my $p = Parse::BBCode->new();
+    my $code = 'some [b]b code[/b]';
+    my $tree = $p->parse($code);
+    # do something with $tree
+    my $rendered = $p->render_tree($tree);
 
 Or if you want to define your own tags:
 
@@ -669,26 +863,11 @@ change. I'm open for any suggestions on how to improve the
 syntax.
 See L<"TODO"> for what might change.
 
-
-I wrote this module because L<HTML::BBCode> is not extendable (or
-I didn't see how) and L<BBCode::Parser> seemed good at the first
-glance but has some issues, for example it says that he following bbode
-
-    [code] foo [b] [/code]
-
-is invalid, while I think you should be able to write unbalanced code
-in code tags.
-Also BBCode::Parser dies if you have invalid code or not-permitted tags,
-but in a forum you'd rather show a partly parsed text then an error
-message.
-
-What I also wanted is an easy syntax to define own tags, ideally - for
-simple tags - as plain text, so you can put it in a configuration file.
-This allows forum admins to add tags easily. Some forums might want
-a tag for linking to perlmonks.org, other forums need other tags.
-
-Another goal was to always output a result and don't die. I might add an
-option which lets the parser die with unbalanced code.
+If you set up the Parse::BBCode object without arguments, the default tags
+are loaded, and any text outside or inside of parseable tags will go through
+a default subroutine which escapes HTML and replaces newlines with <br>
+tags. If you need to change this you can set the options 'url_finder',
+'text_processor' and 'linebreaks'.
 
 =head2 METHODS
 
@@ -708,6 +887,10 @@ Constructor. Takes a hash reference with options as an argument.
         },
         close_open_tags   => 1, # default 0
         strict_attributes => 0, # default 0
+        direct_attributes => 1, # default 1
+        url_finder        => 1, # default 0
+        smileys           => 0, # default 0
+        linebreaks        => 1, # default 1
     );
 
 =over 4
@@ -720,6 +903,55 @@ See L<"TAG DEFINITIONS">
 
 See L<"ESCAPES">
 
+=item url_finder
+
+See L<"URL FINDER">
+
+=item smileys
+
+If you want to replace smileys with an icon:
+
+    my $parser = Parse::BBCode->new({
+            smileys => {
+                base_url => '/your/url/to/icons/',
+                icons => { qw/ :-) smile.png :-( sad.png / },
+                # sprintf format:
+                # first argument url
+                # second argument original text smiley (HTML escaped)
+                format => '<img src="%s" alt="%s">',
+                # if you need the url and text in a different order
+                # see perldoc -f sprintf, e.g.
+                # format => '<img alt="%2$s" src="%1$s">',
+            },
+        });
+
+This subroutine will be applied during the url_finder (or first, if
+url_finder is 0), and the rest will get processed by the text
+procesor (default escaping html and replacing linebreaks).
+
+Smileys are only replaced if surrounded by whitespace or start/end of line/text.
+
+    [b]bold<hr> :-)[/b] :-(
+
+In this example both smileys will be replaced. The first smiley is at the end
+of the text because the text inside [b][/b] is processed on its own.
+
+Open to any suggestions here.
+
+=item linebreaks
+
+The default text processor replaces linebreaks with <br>\n.
+If you don't want this, set 'linebreaks' to 0.
+
+=item text_processor
+
+If you need to add any customized text processing (like smiley parsing (although
+I will probably add builtin smiley support in one of the next versions)), you can
+pass a subroutine here. Note that this subroutine also needs to do HTML
+escaping itself!
+
+See L<"TEXT PROCESSORS">
+
 =item close_open_tags
 
 If set to true (1), it will close open tags at the end or before block tags.
@@ -752,11 +984,31 @@ If set to 0, tag syntax is
 
 =item render
 
-Input: The text to parse
+Input: The text to parse, optional hashref
 
 Returns: the rendered text
 
-    my $parsed = $parser->render($bbcode);
+    my $rendered = $parser->render($bbcode);
+
+You can pass an optional hashref with information you need inside
+of your self-defined rendering subs.
+For example if you want to display code in a codebox with a link to
+download the code you need the id of the article (in a forum) and the number
+of the code tag.
+
+    my $parsed = $parser->render($bbcode, { article_id => 23 });
+    # in the rendering sub:
+        my ($parser, $attr, $content, $attribute_fallback, $tag, $info) = @_;
+        my $article_id = $parser->get_params->{article_id};
+        my $code_id = $tag->get_num;
+        # write downloadlink like
+        # download.pl?article_id=$article_id;code_id=$code_id
+        # in front of the displayed code
+
+See examples/code_download.pl for a complete example of how to set up
+the rendering and how to extract the code from the tree. If run as a CGI
+skript it will give you a dialogue to save the code into a file, including
+a reasonable default filename.
 
 =item parse
 
@@ -774,6 +1026,8 @@ Returns: The rendered text
 
     my $parsed = $parser->render_tree($tree);
 
+You can pass an optional hashref, for explanation see L<"render">
+
 =item forbid
 
     $parser->forbid(qw/ img url /);
@@ -820,14 +1074,6 @@ Here is an example of all the current definition possibilities:
 
     my $p = Parse::BBCode->new({
             tags => {
-                '' => sub {
-                    my ($parser, $attr, $content, $info) = @_;
-                    # for explanation of $info see below
-                    # at "Define subroutine for tag"
-                    my $e = Parse::BBCode::escape_html($content);
-                    $e =~ s/\r?\n|\r/<br>\n/g;
-                    $e
-                },
                 i   => '<i>%s</i>',
                 b   => '<b>%{parse}s</b>',
                 size => '<font size="%a">%{parse}s</font>',
@@ -863,21 +1109,6 @@ The following list explains the above tag definitions:
 
 =over 4
 
-=item Plain text not in tags
-
-This defines how plain text should be rendered:
-
-    '' => sub {
-        my $e = Parse::BBCode::escape_html($_[2]);
-        $e =~ s/\r?\n|\r/<br>\n/g;
-        $e
-    },
-
-In the most cases, you would want HTML escaping like shown above.
-This is the default, so you can leave it out. Only if you want
-to render BBCode into plain text or something else, you need this
-option.
-
 =item C<%s>
 
     i => '<i>%s</i>'
@@ -1110,6 +1341,123 @@ it. It returns the empty string if the input is invalid.
 
 See L<Parse::BBCode::HTML/default_escapes> for the detailed list of escapes.
 
+=head1 URL FINDER
+
+Usually one wants to also create hyperlinks from any url found in the
+bbcode, not only in url tags.
+The following code will use L<URI::Find> to search for all types of
+urls (unless inside of a url tag itself), create a link in the given
+format and html-escape the rest.
+If the url is longer than 50 chars, it will cut the link title and append three dots.
+If you set max_length to 0, the title won't be cut.
+
+    my $p = Parse::BBCode->new({
+            url_finder => {
+                max_length  => 50,
+                # sprintf format:
+                format      => '<a href="%s" rel="nofollow">%s</a>',
+            },
+            tags => ...
+        });
+
+Note: If you use the special tag '' in the tag definitions you will
+overwrite the url finder and have to do that yourself.
+
+Alternative:
+
+    my $p = Parse::BBCode->new({
+            url_finder => 1,
+            ...
+
+This will use the default like shown above (max length 50 chars).
+
+Default is 0.
+
+=head1 TEXT PROCESSORS
+
+If you set url_finder and linebreaks to 1, the default text processor
+will work like this:
+
+    my $post_processor = \&sub_for_escaping_HTML;
+    $text = code_to_replace_urls($text, $post_processor);
+    $text =~ s/\r?\n|\r/<br>\n/g;
+    return $text;
+
+It will be applied to text outside of bbcode and inside of parseable
+bbcode tags (and not to code tags or other tags with unparsed content).
+
+If you need an additional post processor this usually cannot be done
+after the HTML escaping and url finding. So if you write a text processor it
+must do the HTML escaping itself.
+For example if you want to replace smileys with image tags you cannot simply do:
+
+    $text =~ s/ :-\) /<img src=...>/g;
+
+because then the image tag would be HTML escaped after that.
+On the other hand it's usually not possible to do something like
+that *after* the HTML escaping since that might introduce text
+sequences that look like a smiley (or whatever you want to replace).
+
+So a simple example for a customized text processor would be:
+
+    ...
+    url_finder     => 1,
+    linebreaks     => 1,
+    text_processor => sub {
+        # for $info hash description see render() method
+        my ($text, $info) = @_;
+        my $out = '';
+        while ($text =~ s/(.*)( |^)(:\))(?= |$)//mgs) {
+            # match a smiley and anything before
+            my ($pre, $sp, $smiley) = ($1, $2, $3);
+            # escape text and add smiley image tag
+            $out .= Parse::BBCode::escape_html($pre) . $sp . '<img src=...>';
+        }
+        # leftover text
+        $out .= Parse::BBCode::escape_html($text);
+        return $out;
+    },
+
+This will result in:
+Replacing urls, applying your text_processor to the rest of the text and
+after that replace linebreaks with <br> tags.
+
+If you want to completely define the plain text processor yourself (ignoring
+the 'linebreak', 'url_finder', 'smileys' and 'text_processor' options) you define the
+special tag with the empty string:
+
+    my $p = Parse::BBCode->new({
+        tags => {
+            '' => sub {
+                my ($parser, $attr, $content, $info) = @_;
+                return frobnicate($content);
+                # remember to escape HTML!
+            },
+            ...
+
+=head1 WHY ANOTHER BBCODE PARSER
+
+I wrote this module because L<HTML::BBCode> is not extendable (or
+I didn't see how) and L<BBCode::Parser> seemed good at the first
+glance but has some issues, for example it says that the following bbode
+
+    [code] foo [b] [/code]
+
+is invalid, while I think you should be able to write unbalanced code
+in code tags.
+Also BBCode::Parser dies if you have invalid code or not-permitted tags,
+but in a forum you'd rather show a partly parsed text then an error
+message.
+
+What I also wanted is an easy syntax to define own tags, ideally - for
+simple tags - as plain text, so you can put it in a configuration file.
+This allows forum admins to add tags easily. Some forums might want
+a tag for linking to perlmonks.org, other forums need other tags.
+
+Another goal was to always output a result and don't die. I might add an
+option which lets the parser die with unbalanced code.
+
+
 =head1 TODO
 
 =over 4
@@ -1134,14 +1482,18 @@ Attributes always have to look like:
     [tag=main_attribute other=foo]...
     [tag="main_attribute" other="foo"]...
 
-=item Redirects for url tags
+I would like to add support for different syntax, because it might
+happen that one has old bbcode lying around (maybe taken over from
+a different forum software) and cannot manually replace all the invalid
+bbcode.
+
+=item Short bbcode like syntax for urls
+
+Since I need this myself in my board software and currently do the
+replacing in the text processor I would like to add tags like this:
+
+    [cpan://This::Module|title]
 
-In a forum you might want to prefix links and images with a redirect
-script so that the actual referrer will be hidden from the target
-url. This is extremely helpful if you are using session-ids in your
-urls.
-I plan to add an option for url tags which lets you define the
-redirect-script url.
 
 =back
 
@@ -1151,7 +1503,14 @@ perl >= 5.6.1, L<Class::Accessor::Fast>, L<URI::Escape>
 
 =head1 SEE ALSO
 
-L<BBCode::Parser>, L<HTML::BBCode>, L<HTML::BBReverse>
+L<BBCode::Parser> - a parser which supplies the parsed tree if necessary. Too
+strict though for using in forums where people write unbalanced bbcode
+
+L<HTML::BBCode> - simple processor, no parse tree, good enough for processing
+usual bbcode with the most common tags
+
+L<HTML::BBReverse> - really simple proccessor, just replaces start and end tags
+independently by their HTML aequivalents, so not very useful in many cases
 
 See C<examples/compare.html> for a feature comparison of the
 modules and feel free to report mistakes.
@@ -1177,7 +1536,7 @@ Sascha Kiefer
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2008 by Tina Mueller
+Copyright (C) 2011 by Tina Mueller
 
 This library is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself, either Perl version 5.6.1 or, at your option,
