@@ -51,23 +51,64 @@ my %default_tags = (
             my ($parser, $attr, $content, $attribute_fallback, $tag) = @_;
             $$content =~ s/^\n+//;
             $$content =~ s/\n+\z//;
-            return "<ul>$$content</ul>";
+            my $type = "ul";
+            my $style = '';
+            if ($attr) {
+                if ($attr eq '1') {
+                    $type = "ol";
+                }
+                elsif ($attr eq 'a') {
+                    $type = "ol";
+                    $style = ' style="list-style-type: lower-alpha"';
+                }
+            }
+            return "<$type$style>$$content</$type>";
         },
     },
     '*' => {
         parse => 1,
         code => sub {
-            my ($parser, $attr, $content, $attribute_fallback, $tag) = @_;
+            my ($parser, $attr, $content, $attribute_fallback, $tag, $info) = @_;
             $$content =~ s/\n+\z//;
-            return "<li>$$content</li>",
+            if ($info->{stack}->[-2] eq 'list') {
+                return "<li>$$content</li>",
+            }
+            return Parse::BBCode::escape_html($tag->raw_text);
         },
         close => 0,
         class => 'block',
     },
-    'quote' => 'block:<div class="bbcode_quote_header">%{html}a:
-<div class="bbcode_quote_body">%s</div></div>',
-    'code'  => 'block:<div class="bbcode_code_header">%{html}a:
-<div class="bbcode_code_body">%{html}s</div></div>',
+    'quote' => {
+        code => sub {
+            my ($parser, $attr, $content) = @_;
+            my $title = 'Quote';
+            if ($attr) {
+                $title = Parse::BBCode::escape_html($attr);
+            }
+            return <<"EOM";
+<div class="bbcode_quote_header">$title:
+<div class="bbcode_quote_body">$$content</div></div>
+EOM
+        },
+        parse => 1,
+        class => 'block',
+    },
+    'code'  => {
+        code => sub {
+            my ($parser, $attr, $content) = @_;
+            my $title = 'Code';
+            if ($attr) {
+                $title = Parse::BBCode::escape_html($attr);
+            }
+            $content = Parse::BBCode::escape_html($$content);
+            return <<"EOM";
+<div class="bbcode_code_header">$title:
+<div class="bbcode_code_body">$content</div></div>
+EOM
+        },
+        parse => 0,
+        class => 'block',
+    },
     'noparse' => '%{html}s',
 );
 my %optional_tags = (
@@ -129,7 +170,9 @@ sub default_escapes {
 
 sub optional {
     my ($class, @keys) = @_;
-    return @keys ? (grep defined, @optional_tags{@keys}) : %optional_tags;
+    return @keys
+        ? (map { $_ => $optional_tags{$_} } grep  { defined $optional_tags{$_} } @keys)
+        : %optional_tags;
 }
 
 
