@@ -1,12 +1,12 @@
 package Parse::BBCode::Text;
-$Parse::BBCode::Text::VERSION = '0.15_002'; # TRIAL
-
-$Parse::BBCode::Text::VERSION = '0.15002';use strict;
+use strict;
 use warnings;
+use Data::Dumper;
 use Carp qw(croak carp);
 
 use base qw/ Parse::BBCode /;
 
+our $VERSION = '0.03';
 my %default_tags = (
     'b'     => '%s',
     'i'     => '%s',
